<?php

namespace Drupal\content_moderation_permissions;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\content_moderation\StateTransitionValidationInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workflows\StateInterface;
use Drupal\workflows\Transition;
use Drupal\workflows\WorkflowInterface;

/**
 * Validates whether a certain state transition is allowed.
 */
class StateTransitionValidation implements StateTransitionValidationInterface {


  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInfo;

  /**
   * The transition validation service.
   *
   * @var \Drupal\content_moderation\StateTransitionValidationInterface
   */
  protected $transitionValidation;

  /**
   * StateTransitionValidation constructor.
   *
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_info
   *   The moderation information service.
   * @param \Drupal\content_moderation\StateTransitionValidationInterface $transition_validation
   *   The moderation information service.
   */
  public function __construct(ModerationInformationInterface $moderation_info, StateTransitionValidationInterface $transition_validation) {
    $this->moderationInfo = $moderation_info;
    $this->transitionValidation = $transition_validation;
  }

  /**
   * {@inheritdoc}
   */
  public function getValidTransitions(ContentEntityInterface $entity, AccountInterface $user) {
    $workflow = $this->moderationInfo->getWorkflowForEntity($entity);
    $current_state = $entity->moderation_state->value ? $workflow->getTypePlugin()->getState($entity->moderation_state->value) : $workflow->getTypePlugin()->getInitialState($entity);

    $validTransitions = array_filter($current_state->getTransitions(), function (Transition $transition) use ($workflow, $user, $entity) {
      return $user->hasPermission('use ' . $workflow->id() . ' transition ' . $transition->id() . ' for ' . $entity->bundle());
    });

    /** @var \Drupal\workflows\Transition[] $transitions */
    $transitions = $this->transitionValidation->getValidTransitions($entity, $user);

    $validTransitions = array_merge($validTransitions, $transitions);

    return $validTransitions;
  }

  /**
   * {@inheritdoc}
   */
  public function isTransitionValid(WorkflowInterface $workflow, StateInterface $original_state, StateInterface $new_state, AccountInterface $user, ContentEntityInterface $entity = NULL) {
    if ($entity === NULL) {
      @trigger_error('Omitting the $entity parameter from isTransitionValid is deprecated in drupal:8.9.0 and will be required in drupal:9.0.0. $entity parameter will be mandatory on drupal:9.0.0. See https://www.drupal.org/node/3026087', E_USER_DEPRECATED);
    }

    $transition = $workflow->getTypePlugin()->getTransitionFromStateToState($original_state->id(), $new_state->id());
    // Check if the permission is given globally.
    if ($user->hasPermission('use ' . $workflow->id() . ' transition ' . $transition->id())) {
      return $user->hasPermission('use ' . $workflow->id() . ' transition ' . $transition->id());
    }
    return $user->hasPermission('use ' . $workflow->id() . ' transition ' . $transition->id() . ' for ' . $entity->bundle());
  }

}
